
import urllib.parse


class PersonResource:

    def __init__(self, client, config: dict, version: str = "v1"):
        self.client = client
        self.version = version
        self.login = config.get("username")
        self.password = config.get("password")
        self.custom_uri = config.get("custom_uri")

    def find_person_id_by_email(self, email: str):
        """ Find person ID by email

        Args:
            email (str): Person e-mail. Must be formatted as valid e-mail string. Also might need to be properly
            encoded as required by URL syntax standard. For example, the '@' character should be replaced with
            the %40 code, '.' - with %2E and so on.

        Returns:
            personId (int): Person ID

        Example:
            {"personId":910}
        """

        email = urllib.parse.quote(email)
        # replace hyphen(%2D), period(%2E),underscore(%5F), or tilde(%7E), see RFC3986
        email = email.replace("-", "%2D").replace(".", "%2E").replace("_", "%5F").replace("~", "%7E")

        url = f"/person/{self.version}/id/byEmail/{email}"
        headers = {"Accept": "application/json", "login": self.username, "customerUri": self.custom_uri,
                   "password": self.password}
        response = self.client.get(url, headers=headers)
        return response

    def find_person_id(self, person_id: int):
        """ Find person by ID

        Args:
            person_id (str): Person ID

        Returns:
            id (int): Person ID
            organizationID (int): Organization ID
            email (str): Person e-mail
            firstName (str): Person firstname
            middleName (str): Person lastname
            lastName (str): Person middlename
            validationType (str): Person validation type. Values: [STANDARD, HIGH]
            phone (str): Person Phone
            commonName (str): Person CommonName
            secondaryEmails (List[str]): Person Secondary Emails

        Example:
            {"id":909,"firstName":"Tester","middleName":"","lastName":"","email":"14375_nobody@nobody.comodo.od.ua",
            "organizationId":10398,"validationType":"STANDARD","phone":"123456789",
            "secondaryEmails":["321nobody@nobody.comodo.od.ua","123@email.com"],"commonName":"Tester"}
        """

        url = f"/person/{self.version}/{person_id}"
        headers = {"Accept": "application/json;charset=UTF-8", "customerUri": self.custom_uri, "login": self.username,
                   "password": self.password}
        response = self.client.get(url, headers=headers)
        return response

    def create_new_person(self, first_name: str, middle_name: str, last_name: str, email: str, validation_type: str,
                          organization_id: int, phone: str, common_name: None, secondary_emails: [str]):
        """ Create new person

        Args:
            first_name (str): Person's first name [Must not be blank, Size must be between 1 and 64 inclusive]
            last_name (str): Person's middle name [Must not be blank, Size must be between 1 and 64 inclusive]
            middle_name (str): Person's last name [Size must be between 0 and 64 inclusive]
            email (str): Person's email [Must be a well-formed email address, Must not be empty, Size must be between 0 and 128 inclusive]
            validation_type (str): Person’s validation type. Values: [STANDARD, HIGH] [Must not be null]
            organization_id (int): Organization ID [Must be at least 1, Must not be null]
            phone (str): Person Phone [Must match the regular expression: [#|0-9|\(|\)|\-|\+| x]*, Size must be between 0 and 32 inclusive]
            common_name (str): Person commonName [Size must be between 0 and 64 inclusive]
            secondary_emails (List[str]): Person Secondary Emails

        Example:
            {"firstName":"First Name","middleName":"Middle Name","lastName":"Last Name","email":"test@email.com",
            "organizationId":10390,"validationType":"STANDARD","phone":"","secondaryEmails":[],"commonName":null}
        """

        data = {"firstName": first_name, "middleName": middle_name, "lastName": last_name, "email": email,
                "organizationId": organization_id, "validationType": validation_type, "phone": phone,
                "secondaryEmails": secondary_emails, "commonName": common_name}

        url = f"/person/{self.version}"
        headers = {"login": self.username, "customerUri": self.custom_uri,
                   "Content-Type": "application/json;charset=UTF-8", "password": self.password}
        response = self.client.post(url, data=data, headers=headers)
        return response

    def update_person(self, person_id: int, first_name: str, middle_name: str, last_name: str, email: str,
                      validation_type: str, organization_id: int, phone: str, common_name: str,
                      secondary_emails: [str]):
        """ Update person

        Args:
            first_name (str): Person's first name [Must not be blank, Size must be between 1 and 64 inclusive]
            middle_name (str): Person's last name [Size must be between 0 and 64 inclusive]
            last_name (str): Person's middle name [Must not be blank, Size must be between 1 and 64 inclusive]
            email (str): Person's email [Must be a well-formed email address, Must not be empty, Size must be between 0 and 128 inclusive]
            validation_type (str): Person’s validation type. Values: [STANDARD, HIGH] [Must not be null]
            organization_id (int): Organization ID [Must be at least 1, Must not be null]
            phone (str): Person Phone [Must match the regular expression: [#|0-9|\(|\)|\-|\+| x]*, Size must be between 0 and 32 inclusive]
            common_name (str): Person commonName [Size must be between 0 and 64 inclusive]
            secondary_emails (List[str]): Person Secondary Emails

        Example:
            {"firstName":"First Name","middleName":"Middle Name","lastName":"Last Name","email":"test@email.com",
            "organizationId":10420,"validationType":"STANDARD","phone":null,
            "secondaryEmails":["321nobody@nobody.comodo.od.ua","123@email.com"],"commonName":null}
        """

        url = f"/person/{self.version}/{person_id}"

        data = {"firstName": first_name, "middleName": middle_name, "lastName": last_name, "email": email,
                "organizationId": organization_id, "validationType": validation_type, "phone": phone,
                "secondaryEmails": secondary_emails, "commonName": common_name}
        headers = {"login": self.username, "customerUri": self.custom_uri,
                   "Content-Type": "application/json;charset=UTF-8", "password": self.password}

        response = self.client.put(url, data=data, headers=headers)
        return response

    def delete_person(self, person_id):
        """ Delete Person

        Args:
            person_id (int): Person ID being deleted

        """

        url = f"/person/{self.version}/{person_id}"
        headers = {"login": self.username, "customerUri": self.custom_uri, "password": self.password}

        response = self.client.delete(url, headers=headers)
        return response

    def list_persons(self, **parameter):
        """ List Person

        Args:
            position (int): Position shift
            size (int): Count of entries
            name (str): Person name (url encoded)
            organization_id (int): Organization ID
            email (str): Person email
            common_name (str): Person commonName
            phone (str): Person phone
            secondary_email (List[str]): Person Secondary Email

        Returns:
            [] (List[Dict]): List of persons
            [].id (int): Person ID
            [].organizationId (int): Organization ID
            [].email (str): Person e-mail
            [].firstName (str): Person firstname
            [].lastname (str): Person lastname
            [].middlename (str): Person middlename
            [].validationTyle (str): Person validation type
            [].phone (str): Person Phone
            [].commonName (str): Person CommonName
            [secondaryEmails (List[str]): Person Secondary Emails

        Example:
            [{"id":913,"firstName":"Tester","middleName":"","lastName":"","email":"14399_nobody@nobody.comodo.od.ua",
            "organizationId":10406,"validationType":"STANDARD","phone":"123456789",
            "secondaryEmails":["321nobody@nobody.comodo.od.ua","123@email.com"],"commonName":"Tester"}]
        """

        url = f"/person/{self.version}/"
        if parameter:
            key, value = parameter.popitem()
            url = f"{url}?{key}={value}"
            for para in parameter:
                url = f"{url}&{para}={parameter[para]}"

        headers = {"customerUri": self.custom_uri, "login": self.username, "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def import_client_certificate_with_private_key(self, person_id: int, p12: str, password: str, custom_fields: [str]):
        """ Import client certificate with private key for person

        Args:
            p12 (str): Base64 encoded pkcs12 file []
            password (str): Password to access pkcs12 file [Optional]
            customFields (List): An array of custom fields if required [Optional]

        Example:
            {"p12": ""MIIc9AIBAzCCHK...KMCAgQA", "password":"11","customFields":[]}

        Returns:
            id (int): ID of created certificate based on imported payload

        """

        url = f"/person/{self.version}/{person_id}/import-key "
        data = {"p12": p12, "password": password, "customFields": custom_fields}
        headers = {"customerUri": self.custom_uri, "Content-Type": "application/json;charset=UTF-8",
                   "login": self.username, "password": self.password}

        response = self.client.post(url, data=data, headers=headers)
        return response




import urllib.parse


class ClientCertificates:
    """ Client resource is used to perform operation on Client Certificates """

    def __init__(self, client, config: dict, version: str = "v1"):
        self.client = client
        self.version = version
        self.login = config.get("username")
        self.password = config.get("password")
        self.custom_uri = config.get("custom_uri")

    def listing_client_certificate_types(self, ):
        """ A GET request will list all of SSL types.

        Returns:
            [] (List): An array of available Client types
            [].name (str): Name of Client type
            [].id (int): ID of Client type
            [].terms (List): An array of term(days) for Client type

        Example:
            [{"id":7786,"name":"Standard Persona Validated Cert","terms":[365,730,1095,1460,1825]},
            {"id":7787,"name":"High Persona Validated Cert","terms":[365,730,1095,1460,1825]}]
         """

        url = f"/smime/{self.version}/types"
        headers = {"login": self.username, "Accept": "application/json",  "customerUri": self.custom_uri,
                  "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def custom_fields_for_client_certificate(self):
        """ Special fields that enable the administrator to add their own identification reference(s) on Client
            certificates requested through SCM.

        Returns:
            [] (List): An array of custom fields
            [].name (str): Custom field name
            [].mandatory (bool): Is field mandatory
            [].id (int): Custom field id

        Example:
            [{"id":167,"name":"testName","mandatory":true}]
        """

        url = f"/smime/{self.version}/customFields"
        headers = {"Accept": "application/json", "customerUri": self.custom_uri, "login": self.username,
                   "password": self.password}
        response = self.client.get(url, headers=headers)
        return response

    def enroll_client_certificate(self, org_id: int, csr: str, cert_type: int, term: int, email: str, phone: str,
                                  secondary_emails: list, first_name: str, middle_name: str, last_name: str,
                                  custom_fields: list):
        """ Creation and submission of a request for a new Client certificate.

        Args:
            org_id (int): Organization ID [Must be at least 1]
            csr (str): Certificate signing request
            cert_type (int): Certificate type ID [Must be at least 1]
            term (int): Certificate validity period in days [Must be at least 1]
            email (str): Person e-mail [Must be a well-formed email address, Must not be empty, Size must be between 0
                         and 128 inclusive]
            phone (str): Person telephone [Must match the regular expression: [#|0-9|\(|\)|\-|\+| x]*, Size must be
                         between 0 and 32 inclusive]
            secondary_emails (List): Person secondary e-mails
            first_name (str): Person first name [firstName must not be empty, firstName + ' ' + middleName + ' ' +
                              lastName must be in range of 1 to 64 characters]
            middle_name (str): Person middle name
            last_name (str): Person last name [lastName must not be empty, firstName + ' ' + middleName + ' ' + lastName
                             must be in range of 1 to 64 characters]
            custom_fields (List): Custom fields to be applied to requested certificate

        Returns:
            orderNumber (int): Order number

        Example:
            {"orderNumber":16180}
        """

        url = f"/smime/{self.version}/enroll"
        headers = {"login": self.username, "Content-Type": "application/json;charset=utf-8",
                   "customerUri": self.custom_uri, "password": self.password}
        data = {"orgId": org_id, "csr": csr, "certType": cert_type, "term": term, "email": email, "phone": phone,
                "secondaryEmails": secondary_emails, "firstName": first_name, "middleName": middle_name,
                "lastName": last_name, "customFields": custom_fields}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def collect_client_certificate(self, order_number: int):
        """ Delivering the newly issued Client certificate from CA to the administrator for download.

        Args:
            order_number (int): Order number
        """

        url = f"/smime/{self.version}/collect/{order_number}"
        headers = {"login": self.username, "customerUri": self.custom_uri, "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def renew_client_certificate_by_order_number(self, order_number: int):
        """ Submission of a request for a new Client certificate using the CSR and parameters of the initial Client
            certificate. The initial certificate is defined by its order number.

        Args:
            order_number (int): Order number. Must be greater than or equal to 1.
        """

        url = f"/smime/{self.version}/renew/order/{order_number}"
        headers = {"Accept": "application/json", "login": self.username, "customerUri": self.custom_uri,
                   "password": self.password}
        response = self.client.get(url, headers=headers)
        return response

    def renew_client_certificate_by_serial_number(self, serial: int):
        """ Submission of a request for a new Client certificate using the CSR and parameters of the initial Client
            certificate. The initial certificate is defined by its serial number.

        Args:
            serial (int): Serial number.
        """

        url = f"/smime/{self.version}/renew/serial/{serial}"
        headers = {"Accept": "application/json", "customerUri": self.custom_uri, "login": self.username,
                   "password": self.password}

        response = self.client.post(url, headers=headers)
        return response

    def replace_client_certificate_by_order_number(self, order_number: int, csr: str, reason: str, revoke: bool):
        """ Submission of a request for a replace of a Client certificate using new CSR and the parameters of the
            initial Client certificate. The initial certificate is defined by its order number.

        Args:
            order_number (int): Order Number of certificate which you are going to replace.
            csr (str): CSR related to new key pair [Must not be empty, Size must be between 100 and 2147483647 inclusive]
            reason (str): Short message explaining why certificate needs to be replaced [Must not be empty, Size must be
                          between 1 and 512 inclusive]
            revoke (bool): Previous certificate will be revoked if true [Must not be null]
        """

        url = f"/smime/{self.version}/replace/order/{order_number}"
        headers = {"customerUri": self.custom_uri,  "Content-Type": "application/json;charset=utf-8",
                   "login": self.username, "password": self.password}
        data = {"csr": csr, "reason": reason, "revoke": revoke}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def revoke_client_certificate_by_order_number(self, order_number: int, reason: str):
        """ Sending a request to CA to add the particular Client certificate in certificate revocation list.

        Args:
            order_number (int): Order number. Must be greater than or equal to 1.
            reason (str):
        """

        url = f"/smime/{self.version}/revoke/order/{order_number}"
        headers = {"Accept": "application/json", "customerUri": self.custom_uri, "login": self.username,
                   "password": self.password}
        data = {"reason": reason}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def revoke_client_certificate_by_serial_number(self, serial_number: int, reason: str):
        """ Sending a request to CA to add the Client certificate under the particular serial number in certificate
            revocation list.

        Args:
            serial_number (int): Order number. Must be greater than or equal to 1.
            reason (str):
        """

        url = f"/smime/{self.version}/revoke/serial/{serial_number}"
        headers = {"Accept": "application/json", "login": self.username, "customerUri": self.custom_uri,
                   "password": self.password}
        data = {"reason": reason}

        response = self.client.get(url, headers=headers, data=data)
        return response

    def revoke_all_client_certificate_related_to_email(self, reason: str, email: str):
        """ Sending a request to CA to add all Client certificates issued for the person with the particular email
            address in certificate revocation list.

        Args:
            reason (str): Short message explaining why certificate needs to be revoked
            email (str): Person e-mail address
        """

        url = f"/smime/{self.version}/revoke"
        headers = {"Accept": "application/json", "customerUri": self.custom_uri,
                   "Content-Type": "application/json;charset=utf-8", "login": self.username, "password": self.password}
        data = {"email": email, "reason": reason}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def list_client_certificates_by_person_id(self, pid: int):
        """ A GET request will return list of all Client certificates for a person with given ID.

        Args:
            pid (int): Person ID. Must be greater than or equal to 1.

        Returns (v2):
            [] (List): An array of Client certificate properties
            [].id (int): Certificate ID
            [].subject (str): Certificate subject
            [].state (str): Certificate state
            [].order_number (int): Certificate order number
            [].serial_number (str): Certificate serial number

        Example:
            [{"id":1,"subject":"S/MIME Subject string","state":"issued",
            "serialNumber":"C3:DB:6F:88:E7:20:DF:99:71:70:59:FB:D0:2D:29:B0","orderNumber":16190}]

        Returns (v1):
            [] (List): An array of Client certificate properties
            [].id (int): Certificate ID
            [].subject (str): Certificate subject
            [].state (str): Certificate state

        Example:
            [{"id":1,"subject":"S/MIME Subject string","state":"issued"}]
        """

        url = f"/smime/{self.version}/byPersonId/{pid}"
        headers = {"Accept": "application/json", "login": self.username, "customerUri": self.custom_uri,
                   "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def list_client_certificates_by_person_email(self, email: str):
        """ A GET request will return list of all Client certificates for a person with given email.

        Args:
            email (str): Person e-mail. Must be formatted as valid e-mail string. Also might need to be properly encoded
                         as required by URL syntax standard. For example, the '@' character should be replaced with the
                         %40 code, '.' - with %2E and so on.

        Returns (v2):
            [] (List): An array of Client certificate properties
            [].id (int): Certificate ID
            [].subject (str): Certificate subject
            [].state (str): Certificate state
            [].orderNumber (int): Certificate order number
            [].serialNumber (str): Certificate serial number

        Example:
            [{"id":1,"subject":"S/MIME Subject string","state":"issued",
            "serialNumber":"C3:DB:6F:88:E7:20:DF:99:71:70:59:FB:D0:2D:29:B0","orderNumber":16190}]

        Returns (v1):
            [] (List): An array of Client certificate properties
            [].id (int): Certificate ID
            [].subject (str): Certificate subject
            [].state (str): Certificate state

        Example:
            [{"id":1,"subject":"S/MIME Subject string","state":"issued"}]
        """

        email = urllib.parse.quote(email)
        # replace hyphen(%2D), period(%2E),underscore(%5F), or tilde(%7E), see RFC3986
        email = email.replace("-", "%2D").replace(".", "%2E").replace("_", "%5F").replace("~", "%7E")

        url = f"/smime/{self.version}/byPersonEmail/{email}"
        headers = {"Accept": "application/json", "login": self.username, "customerUri": self.custom_uri,
                   "password": self.password}

        response = self.client.get(url, headers=headers)
        return response





class ClientAdministratorResource:

    def __init__(self, client, config: dict, version: str = "v1"):
        self.client = client
        self.version = version
        self.login = config.get("username")
        self.password = config.get("password")
        self.custom_uri = config.get("custom_uri")

    def create_client_admin(self, email: str):
        """ Create client admin’s account.

        Args:
            login (str): Login [Must be null or not blank, Must match the regular expression: ^([a-zA-Z0-9@_\.\-\+\'])*, Must not be blank, Size must be between 0 and 128 inclusive]
            email (str): E-Mail [Must be a well-formed email address, Must be null or not blank, Must not be blank]
            forename (str): Forename [Must be null or not blank, Must match the regular expression: ^((?!.*[>|<|!|@|#|\$|\%|\^|\*|\(|\)|~|\?|/|\|\&|\_|\+|=|\"|:|;|,|\{|\}|\[|\]|||`].*).)*$, Must not be blank, Size must be between 0 and 64 inclusive]
            surname (str): [Must be null or not blank, Must match the regular expression: ^((?!.*[>|<|!|@|#|\$|\%|\^|\*|\(|\)|~|\?|/|\|\&|\_|\+|=|\"|:|;|,|\{|\}|\[|\]|||`].*).)*$, Must not be blank, Size must be between 0 and 64 inclusive]
            title (str): Title or Position
            telephone (str): Telephone Number [Must match the regular expression: [#|0-9|\(|\)|\-|\+| x]*]
            street (str): Street Address
            locality (str): Locality
            state (str): State
            post_code (str): Postal Code [Must match the regular expression: ^[a-zA-Z0-9\s-]{0,10}$]
            country (str): Country
            relationship (str): Relationship [Size must be between 0 and 256 inclusive]
            certificateSerialNumber (str): Authentication Certificate Serial Number
            password (str): Password [Must be null or not blank, Must not be blank]
            privileges (List): Privileges
            credentials.[] (List): Credentials [Must not be empty]
            credentials.[].role (str): Role
            credentials.[].orgId (int): Organization Identifier

        Example:
            {"login":"admin","email":"a@a.com","forename":"Admin","surname":"Adminovich","telephone":"+3804821111111",
            "password":"1234","credentials":[{"role":"RAO_SSL","orgId":12299}]}

        Todo:
            * regex input data
        """

        url = f"/person/{self.version}/id/byEmail/{email}"
        headers = {"Accept": "application/json", "customerUri": self.custom_uri,
                   "login": self.username, "Content-Type": "application/json;charset=utf-8", "password": self.password}
        response = self.client.post(url, headers=headers)
        return response

    def update_client_admin(self, id, login: str, email: str, forename: str, surname: str, title: str, telephone: str,
                            street: str, locality: str, state: str, post_code: str, country: str, relationship: str,
                            certificate_serial_number: str, password: str, privileges: str, credentials: list,
                            credentials_role: str, credentials_org_id: int):
        """ Update client admin’s account.

        Args:
            id (int): ID of client admin being updated
            login (str): Login [Must be null or not blank, Must match the regular expression: ^([a-zA-Z0-9@_\.\-\+\'])*, Must not be blank, Size must be between 0 and 128 inclusive]
            email (str): E-Mail [Must be a well-formed email address, Must be null or not blank, Must not be blank]
            forename (str): Forename [Must be null or not blank, Must match the regular expression: ^((?!.*[>|<|!|@|#|\$|\%|\^|\*|\(|\)|~|\?|/|\|\&|\_|\+|=|\"|:|;|,|\{|\}|\[|\]|||`].*).)*$, Must not be blank, Size must be between 0 and 64 inclusive]
            surname (str): [Must be null or not blank, Must match the regular expression: ^((?!.*[>|<|!|@|#|\$|\%|\^|\*|\(|\)|~|\?|/|\|\&|\_|\+|=|\"|:|;|,|\{|\}|\[|\]|||`].*).)*$, Must not be blank, Size must be between 0 and 64 inclusive]
            title (str): Title or Position
            telephone (str): Telephone Number [Must match the regular expression: [#|0-9|\(|\)|\-|\+| x]*]
            street (str): Street Address
            locality (str): Locality
            state (str): State
            post_code (str): Postal Code [Must match the regular expression: ^[a-zA-Z0-9\s-]{0,10}$]
            country (str): Country
            relationship (str): Relationship [Size must be between 0 and 256 inclusive]
            certificate_serial_number (str): Authentication Certificate Serial Number
            password (str): Password [Must be null or not blank, Must not be blank]
            privileges (List): Privileges
            credentials (List): Credentials [Must not be empty]
            credentials_role (str): Role
            credentials_org_id (int): Organization Identifier

        Example:
           {"login":"admin","email":"a@a.com","forename":"Admin","surname":"Adminovich","telephone":"+3804871111111",
           "password":"1234","privileges":["allowCreate","allowDelete","allowEdit"],
           "credentials":[{"role":"RAO_SSL","orgId":12279}]}

        """

        url = f"/admin/{self.version}/{id}"
        headers = {"customerUri": self.custom_uri, "Accept": "application/json", "login": self.username,
                   "Content-Type": "application/json;charset=utf-8", "password": self.password}

        credentials = [{"role": credentials_role, "orgId": credentials_org_id}]
        data = {"login": login, "email": email, "forename": forename, "surname": surname, "telephone": telephone,
                "password": password, "privileges": privileges, "credentials": credentials}

        response = self.client.put(url, headers=headers, data=data)
        return response

    def delete_client_admin(self, id: int):
        """ Delete client admin’s account.

        Args:
            id (int): ID of client admin being deleted
        """

        url = f"/admin/{self.version}/{id}"
        headers = {"login": self.username, "Content-Type": "application/json;charset=utf-8",
                   "customerUri": self.custom_uri, "password": self.password, "Accept": "application/json"}

        response = self.client.delete(url, headers=headers)
        return response

    def get_client_admins_list(self, size: int, **parameter):
        """ Get list of Client Administrators.

        Args:
            size (int): Count of returned entries
            position (int): Position shift
            login (str): Login filter
            email (str): E-mail filter
            status (str): Status filter
            org_id (str): Organization ID filter

        Returns:
            [] (List): Array of requested client admins
            [].id (int): Client admin ID
            [].login (str): Client admin login
            [].email (str): Client admin E-mail
            [].forename (str): Client admin forename
            [].surname (str): Client admin surname

        Example:
            [{"id":12703,"login":"testadmin_customer18160","forename":"client-admin-18166",
            "surname":"client-admin-18166","email":"TestAdmin_Customer18160@aa.com"},
            {"id":12702,"login":"admindrao_customer18160","forename":"client-admin-18164",
            "surname":"client-admin-18164","email":"AdminDrao_Customer18160@aa.com"},
            {"id":12701,"login":"admin_customer18160","forename":"client-admin-18161",
            "surname":"client-admin-18161","email":"Admin_Customer18160@aa.com"}]
        """

        url = f"/admin/{self.version}/"
        if parameter:
            key, value = parameter.popitem()
            url = f"{url}?{key}={value}"
            for para in parameter:
                url = f"{url}&{para}={parameter[para]}"

        headers = {"login": self.username, "customerUri": self.custom_uri, "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def get_client_admin_details(self, id: int):
        """ Get client admin’s details

        Args:
            id (int): ID of client admin whose details are being requested

        Returns:
            id (int): ID
            status (str): Client admin status. Possible values: 'REQUESTED', 'ACTIVE' and 'AWAITING_ME'
            forename (str): Forename
            surename (str): Surname
            login (str): Login
            email (str): Email
            created (str): Client admin creation date
            modified (str): Client admin last modification date
            lastPasswordChange (str): Last password change date
            credentials[] (List): Array of client admin’s roles
            credentials[].role (str): Client admin’s role
            credentials[].orgId (int): Organization ID
            passwordState (str): Client admin’s password state. Possible values: 'ALIVE', 'EXPIRED' and 'NEVER_EXPIRE'
            passwordExpiryDate (str): Expiration date of Client Admin’s password
            activeState (str): Client admin’s active state. Possible values: 'ACTIVE' and 'SUSPENDED'
            failedAttempts (int): Number of failed attempts
            type (str): Client admin account type. Possible values: 'STANDARD', 'IDP_USER' and 'IDP_TEMPLATE'
            privileges (List): Array of client admin’s privileges

        Example:
            {"id":12763,"status":"Active","forename":"client-admin-18284","surname":"client-admin-18284",
            "login":"admindrao_customer18280","email":"AdminDrao_Customer18280@aa.com","created":"2020-01-15",
            "modified":"2020-01-15","lastPasswordChange":"2020-01-15","credentials":[{"role":"DRAO_SSL","orgId":12303}],
            "passwordState":"ALIVE","passwordExpiryDate":"2020-04-14","activeState":"Active",
            "privileges":["allowEdit","allowCreate"],"failedAttempts":0,"type":"Standard"}
        """

        url = f"/admin/{self.version}/{id}/"
        headers = {"Accept": "application/json", "login": self.username, "password": self.password,
                   "customerUri": self.custom_uri}

        response = self.client.get(url, headers=headers)
        return response

    def get_client_admin_roles(self):
        """ Get roles available for client admin

        Returns:
            [] (List): Roles of Client Admin. Possible values: 'MRAO', 'RAO_SSL', 'RAO_SMIME', 'RAO_DEVICE', 'RAO_CS',
                       'DRAO_SSL', 'DRAO_SMIME', 'DRAO_DEVICE', and 'DRAO_CS'

        Example:
            ["MRAO","RAO_SSL","RAO_SMIME","RAO_CS","RAO_DEVICE","DRAO_SSL","DRAO_SMIME","DRAO_CS","DRAO_DEVICE"]
        """

        url = f"/admin/{self.version}/roles"
        headers = {"customerUri": self.custom_uri, "Accept": "application/json", "login": self.username,
                   "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def get_client_admin_privileges(self, **role):
        """ Get privileges available for client admin

        Args:
            role (dict): Client admin’s role. Multiple roles can be provided.

        Returns:
            [].name (str): Privileges for Client Admin. Possible names: 'allowCreate' - Allow creation of peer admin
                           users, 'allowEdit' - Allow editing of peer admin users, 'allowDelete' - Allow deleting of
                           peer admin users, 'allowDCV' - Allow DCV, 'allowSslChanging' - Allow SSL details changing,
                           'allowSslAutoApprove' - Allow SSL auto approve, 'wsApiUseOnly' - WS API use only,
                           'allowMsAdDiscovery' - MS AD Discovery, 'allowKeyVault' - Allow download keys from Key Vault
            [].description (str): Description for privilege.

        Example:
            [{"name":"allowCreate","description":"Allow creation of peer admin users"},{"name":"allowDelete",
            "description":"Allow deleting of peer admin users"},{"name":"allowEdit",
            "description":"Allow editing of peer admin users"},{"name":"allowSslAutoApprove",
            "description":"Allow SSL auto approve"},{"name":"allowSslChanging",
            "description":"Allow SSL details changing"},{"name":"wsApiUseOnly","description":"WS API use only"}]
        """

        url = f"/admin/{self.version}/privileges"
        if role:
            key, value = role.popitem()
            url = f"{url}?{key}={value}"
            for para in role:
                url = f"{url}&{para}={role[para]}"

    def get_password_state(self, state: str, expiration_date: str):
        """ State of Client Admin’s password

        Returns:
            state (str): State of Client Admin. Possible values: 'ALIVE', 'EXPIRED' and 'NEVER_EXPIRE'
            expiration_date (str): Expiration date of Client Admin’s password

        Example:
            {"expirationDate":"2020-04-14","state":"ALIVE"}
        """

        url = f"/admin/{self.version}/password"
        headers = {"Accept": "application/json", "login": self.username, "customerUri": self.custom_uri,
                   "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def change_password(self, password: str):
        """ Change of Client Admin’s password. Possible only within a month from expiration.

        Args:
            password (str): Password of Client Admin

        Returns (Failed case):
            code (int): Code of error if error has occurred
            description (str): Error message

        Example:
            {"code":-976,"description":"New password must be between 8 and 32 characters."}
        """

        url = f"/admin/{self.version}/changepassword"
        headers = {"Accept": "application/json", "customerUri": self.custom_uri,
                   "Content-Type": "application/json;charset=UTF-8", "login": self.username,
                   "password": self.password}
        data = {"newPassword": password}

        response = self.client.post(url, headers=headers, data=data)
        return response





class SSLCertificates:

    def __init__(self, client, config: dict, version: str = "v1"):
        self.client = client
        self.version = version
        self.login = config.get("username")
        self.password = config.get("password")
        self.custom_uri = config.get("custom_uri")

    def listing_ssl_certificates(self, **parameter):  # TODO
        """ Create client admin’s account.

        Args:
            size (int): Count of returned entries
            position (int): Position shift
            commonName (str): Common Name filter
            subjectAlternativeName (str): Subject Alternative Name filter
            status (str): Status filter. Possible values: 'Invalid', 'Requested', 'Approved', 'Declined', 'Applied',
                          'Issued', 'Revoked', 'Expired', 'Replaced', 'Rejected', 'Unmanaged', 'SAApproved', 'Init'
            discoveryStatus (str): Discovery status filter. Possible values: 'NotDeployed', 'Deployed'
            vendor (str): Vendor filter
            orgId (str): Organization ID filter
            installStatus (str): Install status filter. Possible values: 'NOT_SCHEDULED', 'SCHEDULED', 'STARTED',
                                 'SUCCESSFUL', 'FAILED'
            renewalStatus (str): Renewal status filter. Possible values: 'NOT_SCHEDULED', 'SCHEDULED', 'STARTED',
                                 'SUCCESSFUL', 'FAILED'
            issuer (str): Issuer filter
            serialNumber (str): Serial Number filter
            requester (str): Requester filter
            externalRequester (str): External Requester filter
            signatureAlgorithm (str): Signature Algorithm filter
            keyAlgorithm (str): Key Algorithm filter
            keySize (str): Key Size filter
            sha1Hash (str): SHA1 Hash filter
            md5Hash (str): MD5 Hash filter
            keyUsage (str): Key Usage filter
            extendedKeyUsage (str): Extended Key Usage filter
            requestedVia (str): Requested Via filter. Possible values: 'WEB_FORM', 'CLIENT_ADMIN', 'API', 'DISCOVERY',
                                'IMPORTED', 'SCEP', 'CD_AGENT', 'MS_AGENT', 'MS_CA', 'BULK_REQUEST', 'ACME'

        Returns:
            [] (List): Array of requested ssls
            [].sslId (int): SSL ID
            [].commonName (str): SSL Common Name
            [].subjectAlternativeNames (List): SSL Subject Alternative Names
            [].serialNumber (str): SSL Serial Number

        Example:
            [{"sslId":2417,"commonName":"ccmqa.com"}]
        """

        url = f"/ssl/{self.version}/"
        headers = {"login": self.username, "Content-Type": "application/json",  "customerUri": self.custom_uri,
                   "password": self.password}
        # TODO data ?
        response = self.client.get(url, headers=headers)
        return response

    def listing_ssl_types(self):
        """ List all of SSL types.

        Returns:
            [] (List): An array of available SSL types
            [].name (str): Name of SSL type
            [].id (int): ID of SSL type
            [].terms (List): An array of term(days) for SSL type

        Example:
            [{"id":17973,"name":"EV","terms":[365]}]
        """

        url = f"/ssl/{self.version}/types"
        headers = {"customerUri": self.custom_uri, "Content-Type": "application/json", "login": self.username,
                   "password": self.password}
        response = self.client.get(url, headers=headers)
        return response

    def listing_of_custom_fields_for_ssl(self):
        """ List all of custom fields defined for SSL certificates.

        Returns:
            [] (List): An array of custom fields
            [].name (str): Custom field name
            [].mandatory (bool): Is field mandatory
            [].id (int): Custom field id

        Example:
            [{"id":161,"name":"testName","mandatory":true}]
        """

        url =f"/ssl/{self.version}/customFields"
        headers = {"Accept": "application/json", "login": self.username, "customerUri": self.custom_uri,
                   "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def enroll_ssl_certificate(self, org_id: int, csr, str, subj_alt_names: str, cert_type, number_servers,
                               server_type: int, term: int, comments: str, custom_fields: list,external_requester: str):
        """ Creation and submission of a request for a new SSL certificate.

        Args:
            org_id (int): Organization ID [Must be at least 1,Must not be null]
            csr (str): Certificate signing request [Must not be empty,Size must be between 100 and 2147483647 inclusive]
            subj_alt_names (str): Subject alternative names(comma separated)
            cert_type (int): Certificate type ID
            number_servers (int): Number of Server Licenses. Required for the Wildcard products
            server_type (int): Server type ID [Must be at least 1]
            term (int): Certificate validity period in days [Must be at least 1]
            comments (str): Comments for enroll request [Size must be between 0 and 1024 inclusive]
            custom_fields[] (List): Custom fields to be applied to requested certificate. Must contain mandatory custom
                                    fields.
            custom_fields.name (str): Name of an enabled custom field.
            custom_fields.value (str): Value of the custom field.
            external_requester (str): External Requester. Acceptable format: 'email@domain.com' or 'email1@domain.com,
                                      email2@domain.com' [Size must be between 0 and 512 inclusive]

        Example:
            {"orgId":10557,"subjAltNames":"ccmqa.com","certType":17945,"numberServers":0,"serverType":-1,
            "term":365,"comments":"test","externalRequester":"","customFields":[{"name":"custom field",
            "value":"custom field value"}],"csr":"-----BEGIN CERTIFICATE REQUEST-----\n
            MIIC4jCCAc73.....LWcttIxyWnYgxvwaWX4lfx9A==\n-----END CERTIFICATE REQUEST-----"}

        Returns:
            renewID (str)
            sslId (int)

        Example:
            {"renewId":"3DE7BTmxvkiCGyNa2czs","sslId":2414}
        """

        url = f"/ssl/{self.version}/enroll"
        headers = {"login": self.username, "Content-Type": "application/json", "password": self.password,
                   "customerUri": self.custom_uri}

        data = {"orgId": org_id, "csr": csr, "subjAltNames": subj_alt_names, "certType": cert_type,
                "numberServers": number_servers, "serverType": server_type, "term": term, "comments": comments,
                "customFields": custom_fields, "externalRequester": external_requester}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def enroll_ssl_certificate_with_key_generation(self, org_id: str, common_name: str, subj_alt_names: str,
                                                   cert_type: int, number_servers: int, server_type: int,
                                                   term: int, comments: str, algorithm: str, key_size: int,
                                                   pass_phrase: str, custom_fields: list, external_requester: str):
        """ Creation and submission of a request for a new SSL certificate with generated keypair. Private key will
            be stored inside 'Private Keys Store'.

        Args:
            org_id (int): Organization ID [Must be at least 1,Must not be null]
            common_name (str): Certificate common name [Must not be null,Size must be between 1 and 64 inclusive]
            subj_alt_names (str): Subject alternative names(comma separated)
            cert_type (int): Certificate type ID
            number_servers (int): Number of Server Licenses. Required for the Wildcard products
            server_type (int): Server type ID [Must be at least 1]
            term (int): Certificate validity period in days [Must be at least 1]
            comments (str): Comments for enroll request [Size must be between 0 and 1024 inclusive]
            algorithm (str): Keypair algorithm [Possible values: RSA]
            key_size (int): Keypair key size [Applied only in case keypair algorithm is specified.
                            Possible values: [2048, 4096]]
            pass_phrase (str): Password to protect PKCS#12 certificate. [Size must be between 8 and 32 inclusive]
            custom_fields[] (List): Custom fields to be applied to requested certificate. Must contain mandatory custom
                                    fields.
            custom_fields.name (str): Name of an enabled custom field.
            custom_fields.value (str): Value of the custom field.
            external_requester (str): External Requester. Acceptable format: 'email@domain.com' or 'email1@domain.com,
                                      email2@domain.com' [Size must be between 0 and 512 inclusive]

        Example:
              {"orgId":10559,"subjAltNames":"ccmqa.com","certType":17951,"numberServers":0,"serverType":-1,"term":365,
              "comments":"test","externalRequester":"","customFields":[{"name":"custom field",
              "value":"custom field value"}],"commonName":"ccmqa.com","passPhrase":"password","keySize":2048,
              "algorithm":"RSA"}

        Returns:
            renewID (str)
            sslId (int)

        Example:
            {"renewId":"qIA9MwZk1cDjJqbHp3Oi","sslId":2415}
        """

        url = f"/ssl/{self.version}/enroll-keygen"
        headers = {"customerUri": self.custom_uri, "login": self.username, "Content-Type": "application/json",
                   "password": self.password}

        data = {"orgId": org_id, "commonName": common_name, "subjAltNames": subj_alt_names, "certType": cert_type,
                "numberServers": number_servers, "serverType": server_type, "term": term, "comments": comments,
                "algorithm": algorithm, "keySize": key_size, "passPhrase": pass_phrase, "customFields": custom_fields,
                "externalRequester": external_requester}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def link_to_download_private_key_or_whole_certificate(self, ssl_id: int, format_type: str):
        """ Generation of a link to download private key or entire certificate from Private Key Controller. You will
            need to have enabled authentication certificate to have access to the Private Key Store in order to
            download SSL certificate and the private key.

            This API method requires the User Login via Certificate authentication style.

        Args:
            ssl_id (int): SSL id for which the link will be generated. Positive integer value. Min value 1
            format_type (str): Parameter to specify download format: key only or entire certificate. Possible
            values: 'key' - for Private Key, Base64 encoded, 'p12' - for PKCS#12, Base64 encoded

        Returns:
            link (str)

        Example:
            {"link":"https:/download?token=5NOALU79DN46UBN8CB0PLJFHR1&keyformat=P12"}
        """

        url = f"/ssl/{self.version}/keystore/{ssl_id}/{format_type}"
        headers = {"customerUri": self.custom_uri, "login": self.username, "Content-Type": "application/json",
                   "password": self.password}

        response = self.client.get(url, headers=headers)
        return response

    def collect_ssl_certificate(self, ssl_id: int, format_type: str):
        """ Delivering the newly issued SSL certificate from CA to the administrator for download.

        Args:
            ssl_id (int): Certificate ID. Positive integer value. Min value 1
            format_type (str): Format type for certificate. Allowed values: 'x509' - for Certificate (w/ chain), PEM
                             encoded, 'x509CO' - for Certificate only, PEM encoded, 'base64' - for PKCS#7, PEM encoded,
                             'bin' - for PKCS#7, 'x509IO' - for Root/Intermediate(s) only, PEM encoded, 'x509IOR' - for
                             Intermediate(s)/Root only, PEM encoded, 'pem' - for Certificate (w/ chain), PEM encoded,
                             'pemco' - for Certificate only, PEM encoded
        """

        url = f"/ssl/{self.version}/collect/{ssl_id}/{format_type}"
        headers = {"customerUri": self.custom_uri, "login": self.username, "password": self.password}
        response = self.client.get(url, headers=headers)
        return response

    def revoke_ssl_certificate_by_id(self, ssl_id: int, reason: str):
        """ Sending a request to CA to add the particular SSL certificate in certificate revocation list.

        Args:
            ssl_id (int): Certificate ID
            reason (str): Short message with a reason why certificate needs to be revoked [Must not be empty, Size
                          must be between 0 and 512 inclusive]
        """

        url = f"/ssl/{self.version}/revoke/{ssl_id}"
        headers = {"customerUri": self.custom_uri, "Content-Type": "application/json", "login": self.username,
                   "password": self.password}
        data = {"reason": reason}
        response = self.client.get(url, headers=headers, data=data)
        return response

    def revoke_ssl_certificate_by_serial_number(self, serial_number: int, reason: str):
        """ Sending a request to CA to add the particular SSL certificate in certificate revocation list.

        Args:
            serial_number (int): Serial Number of certificate
            reason (str): Short message with a reason why certificate needs to be revoked [Must not be empty, Size must
                          be between 0 and 512 inclusive]
        """

        url = f"/ssl/{self.version}/revoke/serial/{serial_number}"
        headers = {"login": self.username, "Content-Type": "application/json", "customerUri": self.custom_uri,
                   "password": self.password}
        data = {"reason": reason}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def renew_ssl_certificate_by_renew_id(self, renew_id: int, reason: str):
        """
        Args:
            renew_id (int): Renew ID for certificate. Max length 20 symbols
            reason (str)
        """

        url = f"/ssl/{self.version}/renew/{renew_id}"
        headers = {"login": self.username, "Content-Type": "application/json", "customerUri": self.custom_uri,
                   "password": self.password}
        data = {"reason": reason}

        response = self.client.post(url, headers=headers, data=data)
        return response

    def renew_ssl_certificate_by_id(self, ssl_id: int, csr: str, reason: str, common_name: str,
                                    subject_alternative_names: list):
        """
        Args:
            ssl_id (int): Certificate ID. Positive integer value. Min value 1.
            csr (str): Certificate singing request [Must not be empty, Size must be between 100 and 2147483647 inclusive]
            reason (str): Short message with a reason why certificate needs to be replaced [Must not be empty]
            common_name (str): Certificate common name [Size must be between 1 and 64 inclusive]
            subject_alternative_names (List): Array of subject alternative names

        Example:
            {"csr":"-----BEGIN CERTIFICATE REQUEST-----\nMIIC4jCCAc.....X4lfx9A==\n-----END CERTIFICATE REQUEST-----",
            "reason":"test","commonName":"ccmqa.com","subjectAlternativeNames":["mafia.od.ua"]}
        """

        url = f"/ssl/{self.version}/replace/{ssl_id}"
        headers = {"login": self.username, "Content-Type": "application/json", "password": self.password,
                   "customerUri": self.custom_uri}
        data = {"csr": csr, "reason": reason, "commonName": common_name,
                "subjectAlternativeNames": subject_alternative_names}

        response = self.client.post(url, headers=headers, data=data)
        return response





import geant_tcs_client
import ssl_certificates

client = geant_tcs_client.GEANTTCSClient.connect()

config = {"username": "admin_customer14378", "password": "password123", "custom_uri": "test"}
ssl_certs = ssl_certificates.SSLCertificates(config)

print(ssl_certs.listing_ssl_types())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['geant-tcs-client']

package_data = \
{'': ['*']}

modules = \
['README', 'LICENSE']
install_requires = \
['click>=7.1.2,<8.0.0', 'httpx>=0.17.1,<0.18.0']

entry_points = \
{'console_scripts': ['trackdownchanges = geant-tcs-client.main:main']}

setup_kwargs = {
    'name': 'geant-tcs-client',
    'version': '0.1.1',
    'description': 'Python module for the GÉANT TCS API',
    'long_description': '# GÉANT-TCS-Client (GÉANT Trusted Certificate Service Client)\n\nWork in progress.\n\n# Install\n\n    pip install geant-tcs-client\n\n# Example\n\n    #!/usr/bin/env python\n    \n    from geant_tcs_client import GEANTTCSClient\n    from ssl_certificates import SSLCertificates\n\n    def main():\n\n        client = GEANTTCSClient.connect()\n        \n        config = {"username": "admin_customer14378", "password": "password123", "custom_uri": "test"}\n        ssl_certs = SSLCertificates(config)\n        \n        print(ssl_certs.listing_ssl_types())\n    \n    \n    if __name__ == \'__main__\':\n        main()\n\n',
    'author': 'Robert Grätz',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ikreb7/GEANT-TCS-Client',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '==3.8.5',
}


setup(**setup_kwargs)
